"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("@babel/types");
const util_1 = require("./util");
exports.default = () => {
    const defaultOptions = {
        imports: {
            ignoreLibraries: [],
            ignoreFilenames: '',
            remove: true,
            customImports: [],
        },
        styles: {
            ignoreFilenames: '',
            remove: true,
        },
        propTypes: {
            ignoreFilenames: '',
            remove: true,
            onlyProduction: true,
        }
    };
    let globalOptions = null;
    const STYLE_NAME_SPACE = 'StyleSheet';
    const STYLE_CREATE_METHOD = 'create';
    function remove(path) {
        if (path.parentPath.type === 'ConditionalExpression') {
            path.replaceWith(types_1.unaryExpression('void', types_1.numericLiteral(0)));
        }
        else {
            path.remove();
        }
    }
    function isIgnore(scope, regex) {
        var _a, _b, _c;
        const filename = (_c = (_b = (_a = scope.hub) === null || _a === void 0 ? void 0 : _a.file) === null || _b === void 0 ? void 0 : _b.opts) === null || _c === void 0 ? void 0 : _c.filename;
        if (!filename) {
            return true;
        }
        if (!regex) {
            return false;
        }
        return regex.test(filename);
    }
    function isProptypesRemove(path) {
        const { remove, ignoreFilenames, onlyProduction } = globalOptions.propTypes;
        if (!remove || isIgnore(path.scope, ignoreFilenames)) {
            return false;
        }
        if (onlyProduction) {
            return process.env.NODE_ENV === "production";
        }
        return true;
    }
    const collectNestedObjects = {
        ObjectExpression(path, options) {
            var _a;
            const { identifier, styles } = options;
            const parentPath = path.parentPath;
            if (parentPath.key !== 'init') {
                const propertyKeyName = (_a = parentPath.node.key) === null || _a === void 0 ? void 0 : _a.name;
                if (!propertyKeyName)
                    return;
                const propertyKey = `${identifier} ${propertyKeyName}`;
                if (!styles.has(propertyKey)) {
                    remove(parentPath);
                }
            }
        },
    };
    function normalizeOptions(options) {
        const { imports, styles, propTypes } = options;
        if (imports.customImports) {
            if (!Array.isArray(imports.customImports)) {
                imports.customImports = [imports.customImports];
            }
        }
        else {
            imports.ignoreLibraries = [];
        }
        if (imports.ignoreLibraries) {
            if (!Array.isArray(imports.ignoreLibraries)) {
                imports.ignoreLibraries = [imports.ignoreLibraries];
            }
        }
        else {
            imports.ignoreLibraries = ['react'];
        }
        if (imports.ignoreFilenames) {
            imports.ignoreFilenames = new RegExp(imports.ignoreFilenames.join('|'), 'i');
        }
        else {
            imports.ignoreFilenames = '';
        }
        if (styles.ignoreFilenames) {
            styles.ignoreFilenames = new RegExp(styles.ignoreFilenames.join('|'), 'i');
        }
        else {
            styles.ignoreFilenames = '';
        }
        if (propTypes.ignoreFilenames) {
            propTypes.ignoreFilenames = new RegExp(propTypes.ignoreFilenames.join('|'), 'i');
        }
        else {
            propTypes.ignoreFilenames = '';
        }
    }
    return {
        name: 'rn-remove-unused-code',
        visitor: {
            Program(programPath, state) {
                if (!globalOptions) {
                    const options = state.opts;
                    globalOptions = Object.assign({}, defaultOptions, options);
                    normalizeOptions(globalOptions);
                }
            },
            ImportDeclaration(path) {
                var _a;
                const { node, scope } = path;
                const specifiers = node.specifiers;
                const sourceValue = node.source.value;
                const imports = globalOptions.imports;
                if (isIgnore(path.scope, globalOptions.imports.ignoreFilenames))
                    return;
                if (imports.remove) {
                    for (let i = specifiers.length - 1; i >= 0; i--) {
                        const name = specifiers[i].local.name;
                        const binding = scope.getBinding(name);
                        if (((_a = binding === null || binding === void 0 ? void 0 : binding.referencePaths) === null || _a === void 0 ? void 0 : _a.length) === 0 && !imports.ignoreLibraries.includes(sourceValue)) {
                            remove(binding.path);
                        }
                        if (specifiers.length === 0) {
                            remove(path);
                        }
                    }
                }
                if (imports.customImports.length) {
                    const customImport = imports.customImports.find(imports => imports.libraryName === sourceValue);
                    if (customImport && types_1.isImportSpecifier(specifiers[0])) {
                        const newImports = specifiers.map(specifier => {
                            var _a, _b, _c;
                            const localName = specifier.local.name;
                            const customUrl = (_b = (_a = customImport.customMapping) === null || _a === void 0 ? void 0 : _a[localName]) !== null && _b !== void 0 ? _b : `${(_c = customImport.libraryDirectory) !== null && _c !== void 0 ? _c : 'lib'}/${localName}`;
                            return (types_1.importDeclaration([types_1.importDefaultSpecifier(specifier.local)], types_1.stringLiteral(`${customImport.libraryName}/${customUrl}`)));
                        });
                        path.replaceWithMultiple(newImports);
                    }
                }
            },
            ClassProperty(path) {
                var _a;
                if (!isProptypesRemove(path))
                    return;
                const { node, scope } = path;
                if (((_a = node.key) === null || _a === void 0 ? void 0 : _a.name) === 'propTypes') {
                    const pathClassDeclaration = scope.path;
                    if (util_1.isReactClass(pathClassDeclaration.get('superClass'), scope)) {
                        remove(path);
                    }
                }
            },
            ObjectProperty(path) {
                if (!isProptypesRemove(path))
                    return;
                if (!path.node.computed && path.node.key.name === 'propTypes') {
                    const parent = path.findParent(currentNode => {
                        var _a;
                        if (currentNode.type !== 'CallExpression') {
                            return false;
                        }
                        return ((_a = currentNode.get('callee').node.property) === null || _a === void 0 ? void 0 : _a.name) === 'createClass';
                    });
                    if (parent) {
                        remove(path);
                    }
                    ;
                }
            },
            AssignmentExpression(path) {
                if (!isProptypesRemove(path))
                    return;
                const { node, scope } = path;
                if (node.left.computed || !node.left.property || node.left.property.name !== 'propTypes') {
                    return;
                }
                const className = node.left.object.name;
                const binding = scope.getBinding(className);
                if (!binding)
                    return;
                if (binding.path.isClassDeclaration()) {
                    const superClass = binding.path.get('superClass');
                    if (util_1.isReactClass(superClass, scope)) {
                        remove(path);
                    }
                }
                else if (util_1.isStatelessComponent(binding.path)) {
                    remove(path);
                }
            },
            VariableDeclarator(path) {
                var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
                if (!globalOptions.styles.remove || isIgnore(path.scope, globalOptions.styles.ignoreFilenames))
                    return;
                const { node, scope } = path;
                if (((_c = (_b = (_a = node.init) === null || _a === void 0 ? void 0 : _a.callee) === null || _b === void 0 ? void 0 : _b.object) === null || _c === void 0 ? void 0 : _c.name) === STYLE_NAME_SPACE && ((_f = (_e = (_d = node.init) === null || _d === void 0 ? void 0 : _d.callee) === null || _e === void 0 ? void 0 : _e.property) === null || _f === void 0 ? void 0 : _f.name) === STYLE_CREATE_METHOD) {
                    const source = (_j = (_h = (_g = path.scope.getBinding(STYLE_NAME_SPACE)) === null || _g === void 0 ? void 0 : _g.path) === null || _h === void 0 ? void 0 : _h.parentPath) === null || _j === void 0 ? void 0 : _j.get('source');
                    if (((_k = source === null || source === void 0 ? void 0 : source.node) === null || _k === void 0 ? void 0 : _k.value) !== 'react-native')
                        return;
                    const stylesIdentifier = (_l = node.id) === null || _l === void 0 ? void 0 : _l.name;
                    const binding = scope.getBinding(stylesIdentifier);
                    let hasSkipCollect = false;
                    const stylesCollect = new Set();
                    (_m = binding === null || binding === void 0 ? void 0 : binding.referencePaths) === null || _m === void 0 ? void 0 : _m.forEach(item => {
                        var _a;
                        if ((_a = item.container) === null || _a === void 0 ? void 0 : _a.property) {
                            const propertyKey = `${stylesIdentifier} ${item.container.property.name}`;
                            stylesCollect.add(propertyKey);
                        }
                        else {
                            hasSkipCollect = true;
                        }
                    });
                    if (!hasSkipCollect) {
                        path.traverse(collectNestedObjects, {
                            identifier: stylesIdentifier,
                            styles: stylesCollect
                        });
                    }
                    stylesCollect.clear();
                }
            }
        },
    };
};
//# sourceMappingURL=index.js.map